Option Compare Text
''' <summary>
''' Liste les fichiers dans une liste de fichier
''' </summary>
''' <remarks></remarks>
Public NotInheritable Class ClsFileList
    Inherits ClsBaseThread
    ''' <summary>
    ''' Liste des dossiers  explorer
    ''' </summary>
    ''' <remarks></remarks>
    Private mDirLst As Stack(Of IO.DirectoryInfo) = Nothing
    ''' <summary>
    ''' Liste des fichiers
    ''' </summary>
    ''' <remarks></remarks>
    Private mLstFile As New Queue(Of IO.FileInfo)
    ''' <summary>
    ''' Nombre de dossiers analiss
    ''' </summary>
    ''' <remarks></remarks>
    Private mAnalizedDir As Integer = 0
    ''' <summary>
    ''' Dossier actuel
    ''' </summary>
    ''' <remarks></remarks>
    Private mActDir As String = ""
    ''' <summary>
    ''' Nombre de fichiers analiss
    ''' </summary>
    ''' <remarks></remarks>
    Private mAnalizedFile As Integer = 0
    ''' <summary>
    ''' Mmorise le nombre de millisecondes depuis le dmarrage de l'ordinateur lors du dmarrage de l'analyse
    ''' </summary>
    ''' <remarks></remarks>
    Private mTickStart As Integer = 0

    ''' <summary>
    ''' Constructeur de la classe
    ''' </summary>
    ''' <param name="pDirectoryList">Liste des dossiers  analiser (c:\;f:\Documents)</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal pDirectoryList As List(Of String))
        MyBase.New("FileList")
        mTickStart = Environment.TickCount
        Dim lDirLst As New Stack(Of IO.DirectoryInfo)
        pDirectoryList.Sort()
        For i As Integer = pDirectoryList.Count - 1 To 0 Step -1
            Try
                lDirLst.Push(New IO.DirectoryInfo(pDirectoryList(i)))
            Catch ex As Exception
            End Try
        Next
        mDirLst = lDirLst
    End Sub

#Region "Mthodes overrides"
    ''' <summary>
    ''' Procdure du thread
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overrides Sub ProSubThread()
        Do While mDirLst Is Nothing And MyBase.mCanRun
            Threading.Thread.Sleep(10)
        Loop

        Dim lDir As IO.DirectoryInfo = Nothing

        Dim lLstExtInc As New List(Of String)
        Dim lLstExtExc As New List(Of String)

        'Liste des extensions
        lLstExtInc.AddRange(My.Settings.ExtInc.ToLower.Split(";"c))
        lLstExtExc.AddRange(My.Settings.ExtExc.ToLower.Split(";"c))

        If lLstExtExc.Count = 1 AndAlso lLstExtExc(0) = "" Then lLstExtExc.Clear()

        'Dossiers
        Do While mDirLst.Count > 0 And mCanRun
            lDir = mDirLst.Pop
            mAnalizedDir += 1
            mActDir = lDir.FullName

            Try
                Dim ltDir As New SortedList(Of String, IO.DirectoryInfo)
                Dim llDir As IO.DirectoryInfo() = lDir.GetDirectories

                For Each lObj As IO.DirectoryInfo In llDir
                    Try
                        ltDir.Add(lObj.FullName.ToLower, lObj)
                    Catch ex As Exception
                    End Try
                Next

                For i As Integer = ltDir.Count - 1 To 0 Step -1
                    mDirLst.Push(ltDir.Values(i))
                Next
            Catch ex As Exception
            End Try

            'Fichiers
            Try
                Dim ltDir As New SortedList(Of String, IO.FileInfo)
                Dim llDir As New List(Of IO.FileInfo)

                For Each lStr As String In lLstExtInc
                    llDir.AddRange(lDir.GetFiles(lStr))
                Next


                For Each lObj As IO.FileInfo In llDir
                    Try
                        Dim lBoo As Boolean = True
                        Dim lSt1 As String = lObj.Name.ToLower
                        For Each lStr As String In lLstExtExc
                            If lSt1 Like lStr Then
                                lBoo = False
                                Exit For
                            End If
                        Next
                        If lBoo Then ltDir.Add(lSt1, lObj)
                    Catch ex As Exception
                    End Try
                Next

                SyncLock mLstFile
                    For i As Integer = 0 To ltDir.Count - 1
                        mLstFile.Enqueue(ltDir.Values(i))
                    Next
                End SyncLock
            Catch ex As Exception
            End Try
        Loop

        mActDir = ""
    End Sub
    ''' <summary>
    ''' Gestion des exceptions
    ''' </summary>
    ''' <param name="pEx">Exception gnre</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Protected Overrides Function ProErrThread(ByVal pEx As System.Exception) As Boolean
        MessageBox.Show(pEx.ToString)
        Return True
    End Function
#End Region

#Region "Public ReadOnly Property"
    ''' <summary>
    ''' Retourne le nom du dossier actuel
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ActDir() As String
        Get
            Return mActDir
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nom du fichier qui va suivre
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property ActFile() As String
        Get
            Try
                If mLstFile.Count > 0 Then
                    Return mLstFile.Peek().FullName
                Else
                    Return ""
                End If
            Catch ex As Exception
                Return ""
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nombre total de fichiers reprs
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NbTotalFile() As Integer
        Get
            Return mAnalizedFile + mLstFile.Count
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nombre de fichiers analiss
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NbAnalizedFile() As Integer
        Get
            Return mAnalizedFile
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nombre de fichiers  analyser
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NbRemainingFile() As Integer
        Get
            Return mLstFile.Count
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nombre de dossier total
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NbTotalDir() As Integer
        Get
            Return mAnalizedDir + mDirLst.Count
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nombre de dossiers analyss
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NbAnalizedDir() As Integer
        Get
            Return mAnalizedDir
        End Get
    End Property
    ''' <summary>
    ''' Retourne le nombre de dossiers  analyser
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property NbRemainingDir() As Integer
        Get
            Return mDirLst.Count
        End Get
    End Property
    ''' <summary>
    ''' Retourne le temps total des fichiers reprs
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimeTotalFile() As Integer
        Get
            Try
                Return CInt(CDbl(Environment.TickCount - mTickStart) * (mAnalizedFile + mLstFile.Count) / (mAnalizedFile * 1000))
            Catch ex As Exception
                Return -1
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Retourne le temps des fichiers analiss
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimeAnalizedFile() As Integer
        Get
            Try
                Return CInt(CDbl(Environment.TickCount - mTickStart) / 1000)
            Catch ex As Exception
                Return -1
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Retourne le temps des fichiers  analyser
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimeRemainingFile() As Integer
        Get
            Try
                Return CInt(CDbl(Environment.TickCount - mTickStart) * mLstFile.Count / (mAnalizedFile * 1000))
            Catch ex As Exception
                Return -1
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Retourne le temps des dossier total
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimeTotalDir() As Integer
        Get
            Try
                Return CInt(CDbl(Environment.TickCount - mTickStart) * (mAnalizedDir + mDirLst.Count) / (mAnalizedDir * 1000))
            Catch ex As Exception
                Return -1
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Retourne le temps des dossiers analyss
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimeAnalizedDir() As Integer
        Get
            Try
                Return CInt(CDbl(Environment.TickCount - mTickStart) / 1000)
            Catch ex As Exception
                Return -1
            End Try
        End Get
    End Property
    ''' <summary>
    ''' Retourne le temps des dossiers  analyser
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property TimeRemainingDir() As Integer
        Get
            Try
                Return CInt(CDbl(Environment.TickCount - mTickStart) * mDirLst.Count / (mAnalizedDir * 1000))
            Catch ex As Exception
                Return -1
            End Try
        End Get
    End Property
#End Region

    ''' <summary>
    ''' Rcupre le nombre de fichier pNb si possible
    ''' </summary>
    ''' <param name="pNb">Nombre de fichier  rcuprer</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetFile(ByVal pNb As Integer) As Queue(Of IO.FileInfo)
        Dim lQueue As New Queue(Of IO.FileInfo)

        SyncLock mLstFile
            Do While mLstFile.Count > 0 And lQueue.Count < pNb
                lQueue.Enqueue(mLstFile.Dequeue)
            Loop
        End SyncLock

        mAnalizedFile += lQueue.Count

        Return lQueue
    End Function
    ''' <summary>
    ''' Retourne un fichier en attente
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetFile() As IO.FileInfo
        Dim lQueue As Queue(Of IO.FileInfo) = GetFile(1)
        If lQueue.Count = 0 Then
            Return Nothing
        Else
            Return lQueue.Dequeue
        End If
    End Function
End Class
